@@ -32,6 +32,7 @@ from os_brick.i18n import _
 from os_brick import initiator
 from os_brick.initiator.connectors import base
 from os_brick.initiator.connectors import base_iscsi
+from os_brick.initiator import utils as initiator_utils
 from os_brick import utils
 
 synchronized = lockutils.synchronized_with_prefix('os-brick-')
@@ -109,7 +110,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         """Get iSCSI session information as a list of tuples.
 
         Uses iscsiadm -m session and from a command output like
-            tcp: [1] 192.168.121.250:3260,1 iqn.2010-10.org.openstack:volume-
+            tcp: [1] 192.168.121.250:3260,1 iqn.2010-10.org.openstack:
+            volume- (non-flash)
 
         This method will drop the node type and return a list like this:
             [('tcp:', '1', '192.168.121.250:3260', '1',
@@ -117,9 +119,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         """
         out, err = self._run_iscsi_session()
         if err:
-            LOG.warning("Couldn't find iscsi sessions because "
-                        "iscsiadm err: %s", err)
-            return []
+            LOG.warning("iscsiadm stderr output when getting sessions: %s",
+                        err)
 
         # Parse and clean the output from iscsiadm, which is in the form of:
         # transport_name: [session_id] ip_address:port,tpgt iqn node_type
@@ -135,7 +136,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
     def _get_iscsi_nodes(self):
         """Get iSCSI node information (portal, iqn) as a list of tuples.
 
-        Uses iscsi_adm -m node and from a command output like
+        Uses iscsiadm -m node and from a command output like
             192.168.121.250:3260,1 iqn.2010-10.org.openstack:volume
 
         This method will drop the tpgt and return a list like this:
@@ -175,7 +176,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         :param connection_properties: The dictionary that describes all
                                       of the target volume attributes.
         :type connection_properties: dict
-        :param discover: Wheter doing an iSCSI discovery is acceptable.
+        :param discover: Whether doing an iSCSI discovery is acceptable.
         :type discover: bool
         :returns: list of tuples of (ip, iqn, lun)
         """
@@ -194,6 +195,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         except exception.TargetPortalsNotFound:
             raise
         except Exception:
+            LOG.exception('Exception encountered during portal discovery')
             if 'target_portals' in connection_properties:
                 raise exception.TargetPortalsNotFound(
                     target_portals=connection_properties['target_portals'])
@@ -346,7 +348,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         ip, port = connection_properties['target_portal'].rsplit(':', 1)
         # NOTE(geguileo): I don't know if IPv6 will be reported with []
         # or not, so we'll make them optional.
-        ip = ip.replace('[', '\[?').replace(']', '\]?')
+        ip = ip.replace('[', r'\[?').replace(']', r'\]?')
         out = self._run_iscsiadm_bare(['-m', 'discoverydb',
                                        '-o', 'show',
                                        '-P', 1])[0] or ""
@@ -472,7 +474,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         volume_paths = self.get_volume_paths(connection_properties)
         LOG.info("Found paths for volume %s", volume_paths)
         if volume_paths:
-            return self._linuxscsi.extend_volume(volume_paths)
+            return self._linuxscsi.extend_volume(
+                volume_paths, use_multipath=self.use_multipath)
         else:
             LOG.warning("Couldn't find any volume paths on the host to "
                         "extend volume for %(props)s",
@@ -515,6 +518,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             with excutils.save_and_reraise_exception():
                 self._cleanup_connection(connection_properties, force=True)
 
+    @utils.retry(exceptions=(exception.VolumeDeviceNotFound))
     def _get_device_link(self, wwn, device, mpath):
         # These are the default symlinks that should always be there
         if mpath:
@@ -568,10 +572,12 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                 for __ in range(10):
                     wwn = self._linuxscsi.get_sysfs_wwn(found_devs)
                     if wwn:
-                        return self._get_connect_result(connection_properties,
-                                                        wwn, found_devs)
+                        break
                     time.sleep(1)
-                LOG.debug('Could not find the WWN for %s.', found_devs[0])
+                else:
+                    LOG.debug('Could not find the WWN for %s.', found_devs[0])
+                return self._get_connect_result(connection_properties,
+                                                wwn, found_devs)
 
             # If we failed we must cleanup the connection, as we could be
             # leaving the node entry if it's not being used by another device.
@@ -618,8 +624,16 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         device = hctl = None
         portal = props['target_portal']
         session, manual_scan = self._connect_to_iscsi_portal(props)
-        do_scans = rescans > 0
-        retry = 1
+        do_scans = rescans > 0 or manual_scan
+        # Scan is sent on connect by iscsid, but we must do it manually on
+        # manual scan mode.  This scan cannot count towards total rescans.
+        if manual_scan:
+            num_rescans = -1
+            seconds_next_scan = 0
+        else:
+            num_rescans = 0
+            seconds_next_scan = 4
+
         if session:
             data['num_logins'] += 1
             LOG.debug('Connected to %s', portal)
@@ -628,11 +642,12 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                     if not hctl:
                         hctl = self._linuxscsi.get_hctl(session,
                                                         props['target_lun'])
-                    # Scan is sent on connect by iscsid, so skip first rescan
-                    # but on manual scan mode we have to do it ourselves.
                     if hctl:
-                        if retry > 1 or manual_scan:
+                        if seconds_next_scan <= 0:
+                            num_rescans += 1
                             self._linuxscsi.scan_iscsi(*hctl)
+                            # 4 seconds on 1st rescan, 9s on 2nd, 16s on 3rd
+                            seconds_next_scan = (num_rescans + 2) ** 2
 
                         device = self._linuxscsi.device_name_by_hctl(session,
                                                                      hctl)
@@ -642,11 +657,12 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                 except Exception:
                     LOG.exception('Exception scanning %s', portal)
                     pass
-                retry += 1
-                do_scans = (retry <= rescans and
+                do_scans = (num_rescans <= rescans and
                             not (device or data['stop_connecting']))
                 if do_scans:
-                    time.sleep(retry ** 2)
+                    time.sleep(1)
+                    seconds_next_scan -= 1
+
             if device:
                 LOG.debug('Connected to %s using %s', device,
                           strutils.mask_password(props))
@@ -696,6 +712,13 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         for ip, iqn, lun in ips_iqns_luns:
             props = connection_properties.copy()
             props.update(target_portal=ip, target_iqn=iqn, target_lun=lun)
+
+            # NOTE(yenai): The method _connect_vol is used for parallelize
+            # logins, we shouldn't give these arguments; and it will make a
+            # mess in the debug message in _connect_vol. So, kick them out:
+            for key in ('target_portals', 'target_iqns', 'target_luns'):
+                props.pop(key, None)
+
             threads.append(executor.Thread(target=self._connect_vol,
                                            args=(retries, props, data)))
         for thread in threads:
@@ -845,10 +868,12 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         :type ignore_errors: bool
         """
         return self._cleanup_connection(connection_properties, force=force,
-                                        ignore_errors=ignore_errors)
+                                        ignore_errors=ignore_errors,
+                                        device_info=device_info)
 
     def _cleanup_connection(self, connection_properties, ips_iqns_luns=None,
-                            force=False, ignore_errors=False):
+                            force=False, ignore_errors=False,
+                            device_info=None):
         """Cleans up connection flushing and removing devices and multipath.
 
         :param connection_properties: The dictionary that describes all
@@ -864,6 +889,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         :param ignore_errors: When force is True, this will decide whether to
                               ignore errors or raise an exception once finished
                               the operation.  Default is False.
+        :param device_info: Attached device information.
         :type ignore_errors: bool
         """
         exc = exception.ExceptionChainer()
@@ -880,9 +906,14 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         remove_devices = set()
         for remove, __ in devices_map.values():
             remove_devices.update(remove)
-        multipath_name = self._linuxscsi.remove_connection(remove_devices,
-                                                           self.use_multipath,
-                                                           force, exc)
+
+        path_used = self._linuxscsi.get_dev_path(connection_properties,
+                                                 device_info)
+        was_multipath = (path_used.startswith('/dev/dm-') or
+                         'mpath' in path_used)
+        multipath_name = self._linuxscsi.remove_connection(
+            remove_devices, self.use_multipath, force, exc,
+            path_used, was_multipath)
 
         # Disconnect sessions and remove nodes that are left without devices
         disconnect = [conn for conn, (__, keep) in devices_map.items()
@@ -1012,6 +1043,11 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                                     'node.session.scan', 'manual',
                                     check_exit_code=False)
         manual_scan = not res[1]
+        # Update global indicator of manual scan support used for
+        # shared_targets locking so we support upgrading open iscsi to a
+        # version supporting the manual scan feature without restarting Nova
+        # or Cinder.
+        initiator_utils.ISCSI_SUPPORTS_MANUAL_SCAN = manual_scan
 
         if connection_properties.get('auth_method'):
             self._iscsiadm_update(connection_properties,
@@ -1032,7 +1068,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             for s in sessions:
                 # Found our session, return session_id
                 if (s[0] in self.VALID_SESSIONS_PREFIX and
-                        portal == s[2] and s[4] == target_iqn):
+                        portal.lower() == s[2].lower() and s[4] == target_iqn):
                     return s[1], manual_scan
 
             try:
@@ -1072,7 +1108,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
 
     def _run_iscsi_session(self):
         (out, err) = self._run_iscsiadm_bare(('-m', 'session'),
-                                             check_exit_code=[0, 1, 21, 255])
+                                             check_exit_code=[0, 21, 255])
         LOG.debug("iscsi session list stdout=%(out)s stderr=%(err)s",
                   {'out': out, 'err': err})
         return (out, err)
@@ -1143,7 +1179,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                 # as discovering target.
                 # So target_iqn is updated, and other values aren't updated.
                 recover_connection = copy.deepcopy(connection_properties)
-                recover_connection['target_iqn'] = [iqn]
+                recover_connection['target_iqn'] = iqn
                 self._iscsiadm_update(recover_connection,
                                       "node.startup",
                                       old_node_startup)
